@@ -5,6 +5,7 @@ DEFINE_TEST(test_extract_cpio_bz2)
 DEFINE_TEST(test_extract_cpio_grz)
 DEFINE_TEST(test_extract_cpio_gz)
 DEFINE_TEST(test_extract_cpio_lrz)
+DEFINE_TEST(test_extract_cpio_lz4)
 DEFINE_TEST(test_extract_cpio_lz)
 DEFINE_TEST(test_extract_cpio_lzma)
 DEFINE_TEST(test_extract_cpio_lzo)
@@ -12,6 +13,7 @@ DEFINE_TEST(test_extract_cpio_xz)
 DEFINE_TEST(test_extract_cpio_Z)
 DEFINE_TEST(test_format_newc)
 DEFINE_TEST(test_gcpio_compat)
+DEFINE_TEST(test_missing_file)
 DEFINE_TEST(test_option_0)
 DEFINE_TEST(test_option_a)
 DEFINE_TEST(test_option_b64encode)
@@ -26,9 +28,11 @@ DEFINE_TEST(test_option_J_upper)
 DEFINE_TEST(test_option_l)
 DEFINE_TEST(test_option_lrzip)
 DEFINE_TEST(test_option_L_upper)
+DEFINE_TEST(test_option_lz4)
 DEFINE_TEST(test_option_lzma)
 DEFINE_TEST(test_option_lzop)
 DEFINE_TEST(test_option_m)
+DEFINE_TEST(test_option_passphrase)
 DEFINE_TEST(test_option_t)
 DEFINE_TEST(test_option_u)
 DEFINE_TEST(test_option_uuencode)
