@@ -22,12 +22,14 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $FreeBSD: src/lib/libarchive/config_freebsd.h,v 1.15 2008/09/30 03:53:03 kientzle Exp $
+ * $FreeBSD: head/lib/libarchive/config_freebsd.h 201079 2009-12-28 02:01:42Z kientzle $
  */
 
 /* FreeBSD 5.0 and later have ACL and extattr support. */
 #if __FreeBSD__ > 4
 #define	HAVE_ACL_CREATE_ENTRY 1
+#define	HAVE_ACL_GET_FD_NP 1
+#define	HAVE_ACL_GET_LINK_NP 1
 #define	HAVE_ACL_GET_PERM_NP 1
 #define	HAVE_ACL_INIT 1
 #define	HAVE_ACL_SET_FD 1
@@ -42,6 +44,15 @@
 #define	HAVE_SYS_EXTATTR_H 1
 #endif
 
+#ifdef WITH_OPENSSL
+#define	HAVE_OPENSSL_MD5_H 1
+#define	HAVE_OPENSSL_RIPEMD_H 1
+#define	HAVE_OPENSSL_SHA_H 1
+#define	HAVE_SHA384 1
+#define	HAVE_SHA512 1
+#endif
+
+#define	HAVE_BSDXML_H 1
 #define	HAVE_BZLIB_H 1
 #define	HAVE_CHFLAGS 1
 #define	HAVE_CHOWN 1
@@ -52,6 +63,7 @@
 #define	HAVE_DECL_STRERROR_R 1
 #define	HAVE_DECL_UINT32_MAX 1
 #define	HAVE_DECL_UINT64_MAX 1
+#define	HAVE_DIRENT_H 1
 #define	HAVE_EFTYPE 1
 #define	HAVE_EILSEQ 1
 #define	HAVE_ERRNO_H 1
@@ -66,25 +78,25 @@
 #define	HAVE_FTRUNCATE 1
 #define	HAVE_FUTIMES 1
 #define	HAVE_GETEUID 1
+#define	HAVE_GETGRGID_R 1
 #define	HAVE_GETPID 1
+#define	HAVE_GETPWUID_R 1
 #define	HAVE_GRP_H 1
 #define	HAVE_INTTYPES_H 1
 #define	HAVE_LCHFLAGS 1
 #define	HAVE_LCHMOD 1
 #define	HAVE_LCHOWN 1
 #define	HAVE_LIMITS_H 1
+#define	HAVE_LINK 1
+#define	HAVE_LSTAT 1
 #define	HAVE_LUTIMES 1
 #define	HAVE_MALLOC 1
 #define	HAVE_MD5 1
 #define	HAVE_MD5_H 1
 #define	HAVE_MEMMOVE 1
-#define	HAVE_MEMSET 1
 #define	HAVE_MKDIR 1
 #define	HAVE_MKFIFO 1
 #define	HAVE_MKNOD 1
-#define	HAVE_OPENSSL_MD5_H 1
-#define	HAVE_OPENSSL_RIPEMD_H 1
-#define	HAVE_OPENSSL_SHA_H 1
 #define	HAVE_PIPE 1
 #define	HAVE_POLL 1
 #define	HAVE_POLL_H 1
@@ -97,8 +109,6 @@
 #define	HAVE_SHA1 1
 #define	HAVE_SHA256 1
 #define	HAVE_SHA256_H 1
-#define	HAVE_SHA384 1
-#define	HAVE_SHA512 1
 #define	HAVE_SIGNAL_H 1
 #define	HAVE_STDINT_H 1
 #define	HAVE_STDLIB_H 1
@@ -114,12 +124,18 @@
 #define	HAVE_STRUCT_STAT_ST_BIRTHTIMESPEC_TV_NSEC 1
 #define	HAVE_STRUCT_STAT_ST_FLAGS 1
 #define	HAVE_STRUCT_STAT_ST_MTIMESPEC_TV_NSEC 1
+#define	HAVE_STRUCT_TM_TM_GMTOFF 1
+#define	HAVE_SYMLINK 1
+#define	HAVE_SYS_CDEFS_H 1
 #define	HAVE_SYS_IOCTL_H 1
+#define	HAVE_SYS_MOUNT_H 1
+#define	HAVE_SYS_PARAM_H 1
 #define	HAVE_SYS_SELECT_H 1
 #define	HAVE_SYS_STAT_H 1
 #define	HAVE_SYS_TIME_H 1
 #define	HAVE_SYS_TYPES_H 1
 #undef	HAVE_SYS_UTIME_H
+#define	HAVE_SYS_UTSNAME_H 1
 #define	HAVE_SYS_WAIT_H 1
 #define	HAVE_TIMEGM 1
 #define	HAVE_TZSET 1
@@ -136,7 +152,6 @@
 #define	HAVE_WMEMCMP 1
 #define	HAVE_WMEMCPY 1
 #define	HAVE_ZLIB_H 1
-#define	STDC_HEADERS 1
 #define	TIME_WITH_SYS_TIME 1
 
 /* FreeBSD 4 and earlier lack intmax_t/uintmax_t */
