@@ -1,5 +1,6 @@
 DEFINE_TEST(test_acl_freebsd_nfs4)
-DEFINE_TEST(test_acl_freebsd_posix1e)
+DEFINE_TEST(test_acl_freebsd_posix1e_restore)
+DEFINE_TEST(test_acl_freebsd_posix1e_read)
 DEFINE_TEST(test_acl_nfs4)
 DEFINE_TEST(test_acl_pax)
 DEFINE_TEST(test_acl_posix1e)
@@ -260,6 +261,7 @@ DEFINE_TEST(test_read_format_mtree_nomagic_v1_form)
 DEFINE_TEST(test_read_format_mtree_nomagic_v2_form)
 DEFINE_TEST(test_read_format_mtree_nomagic_v2_netbsd_form)
 DEFINE_TEST(test_read_format_mtree_nonexistent_contents_file)
+DEFINE_TEST(test_read_format_mtree_crash747)
 DEFINE_TEST(test_read_format_pax_bz2)
 DEFINE_TEST(test_read_format_rar_basic)
 DEFINE_TEST(test_read_format_rar_subblock)
@@ -321,6 +323,8 @@ DEFINE_TEST(test_read_format_zip_filename_CP866_CP1251_win)
 DEFINE_TEST(test_read_format_zip_filename_KOI8R_CP1251)
 DEFINE_TEST(test_read_format_zip_filename_UTF8_CP1251)
 DEFINE_TEST(test_read_format_zip_filename_KOI8R_UTF8_2)
+DEFINE_TEST(test_read_format_zip_high_compression)
+DEFINE_TEST(test_read_format_zip_high_compression2)
 DEFINE_TEST(test_read_format_zip_mac_metadata)
 DEFINE_TEST(test_read_format_zip_malformed)
 DEFINE_TEST(test_read_format_zip_msdos)
@@ -379,6 +383,10 @@ DEFINE_TEST(test_write_disk_lookup)
 DEFINE_TEST(test_write_disk_mac_metadata)
 DEFINE_TEST(test_write_disk_no_hfs_compression)
 DEFINE_TEST(test_write_disk_perms)
+DEFINE_TEST(test_write_disk_secure744)
+DEFINE_TEST(test_write_disk_secure745)
+DEFINE_TEST(test_write_disk_secure746a)
+DEFINE_TEST(test_write_disk_secure746b)
 DEFINE_TEST(test_write_disk_secure)
 DEFINE_TEST(test_write_disk_sparse)
 DEFINE_TEST(test_write_disk_symlink)
